from setuptools import setup, find_packages

from pathlib import Path
from oba import __version__ as version

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='oba',
    version=version,
    keywords=['dict', 'object'],
    description='make iter object easy to access (item to attr)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT Licence',
    url='https://github.com/Jyonn/Oba',
    author='Jyonn Liu',
    author_email='i@6-79.cn',
    platforms='any',
    packages=find_packages(),
    install_requires=[
        'nestify'
    ],
)
